classdef USB_Sensor_Simulink_Object < matlab.System & matlab.system.mixin.Propagates
    
    
    properties % Public, tunable properties

    end
    
    properties(Nontunable)
        SerialNumber = 123456;
        deviceStatus = 0;
        deviceStatusChr = '';
        deviceHandle = '';
        instr;
        unitCodeData = {'atm'; 'bar'; 'dyn'; 'ft-H20'; 'ft-lb'; 'g'; 'g-cm'; 'g-mm'; 'in-H20'; 'in-lb';
             'in-oz'; 'kdyn'; 'kg'; 'kg-cm'; 'kg/cm2'; 'kg-m'; 'klbs'; 'kN'; 'kPa'; 'kpsi';
             'lbs'; 'Mdyn'; 'mmHG'; 'mN-m'; 'MPa'; 'MT'; 'N'; 'N-cm'; 'N-m'; 'N-mm';
             'oz'; 'psi'; 'Pa'; 'T'; 'mV/V'; 'A'; 'mA'; 'A'; 'mm'; 'cm';
             'dm'; 'm'; 'km'; 'in'; 'ft'; 'yd'; 'mi'; 'g'; 'mg'; 'LT';
             'mbar'; '?C'; '?F'; 'K'; '?Ra'; 'kN-m'; 'g-m'; 'nV'; 'V'; 'mV';
             'V'; 'kV'; 'NONE'};
    end

    properties(DiscreteState)

    end

    % Pre-computed constants
    properties(Access = private)
        temp = 0;
        offsetValue = 0;
        fullScaleValue = 0;
        fullScaleLoad = 0;
        decimalPoint = 0;
        normalData = 0;
        calculatedReading = 0;
        unitCode = 0;
        unitChr = '';
           
    end
    methods (Static, Access = protected)
      function groups = getPropertyGroupsImpl
         upperGroup = matlab.system.display.Section(...
              'Title', 'Serial Number', ...
              'PropertyList',{'SerialNumber'});
           
         thresholdGroup = matlab.system.display.SectionGroup(...
              'Title', 'Parameters', ...
              'Sections',  [upperGroup]); %#ok<NBRAK>
           
         groups = [thresholdGroup]; %#ok<NBRAK>
      end
   end

    methods(Access = protected)

        function setupImpl(obj)
            % Perform one-time calculations, such as computing constants
            coder.extrinsic('which');
            coder.extrinsic('NET.addAssembly');
            NET.addAssembly(which('FUTEK_USB_DLL.dll'));
            import FUTEK_USB_DLL.*
            import FUTEK_USB_DLL.USB_DLL.*
            obj.instr = USB_DLL();
            %Intializes the connection to instrument with the specified Serial Number.
            %Fetches Device Status to determine if connection is successful or returns
            %the error code and exits
            obj.instr.Open_Device_Connection(int2str(obj.SerialNumber));
            obj.deviceStatus = obj.instr.DeviceStatus();
            if obj.deviceStatus == 0
                disp('Pass')
            else
                obj.deviceStatusChr = num2str(obj.deviceStatus);
                disp(['Device Error ' obj.deviceStatusChr])
                set_param(gcs, 'SimulationCommand', 'stop')
                return;
            end;

                %Fetches the device handle needed to retrieve stored values used in load
                %calculation
                obj.deviceHandle = obj.instr.DeviceHandle();
        end

        function sensorOutput = stepImpl(obj)
            % Implement algorithm. Calculate sensor output for this
            % instrument
    
            %Uses the FUTEK USB DLL to retrieve values for load calculation
            %The .dll returns these values as a System.String
            %This is then converted to a char value and then a number
            %Lastly if there was a retrieval error the .dll returns 'Error' which
            %converts to an empty value. isEmpty() test for that and continues
            %retrieval until a number is returned.
    
            % Offset Value
            while true
                obj.temp = str2num(char(obj.instr.Get_Offset_Value(obj.deviceHandle))); %#ok<*ST2NM>
                if ~(isempty(obj.temp))
                    break
                end
            end
            obj.offsetValue = obj.temp;
    
            %Full Scale Value
            while true
                obj.temp = str2num(char(obj.instr.Get_Fullscale_Value(obj.deviceHandle)));
                if ~(isempty(obj.temp))
                    break
                end
            end
            obj.fullScaleValue = obj.temp;
    
            % Full Scale Load
            while true
                obj.temp = str2num(char(obj.instr.Get_Fullscale_Load(obj.deviceHandle)));
                if ~(isempty(obj.temp))
                    break
                end
            end
            obj.fullScaleLoad = obj.temp;
   
            % Decimal Point
            while true
                obj.temp = str2num(char(obj.instr.Get_Decimal_Point(obj.deviceHandle)));
                if ~(isempty(obj.temp))
                    break
                end
            end
            obj.decimalPoint = obj.temp;
            %Test to see if decimal point returned is outside defined decimal point
            %values
            %See here for more information: 
            %http://www.futek.com/files/docs/API/FUTEK_USB_DLL/webframe.html#DecimalPointCodes.html
            
            if obj.decimalPoint >= 6
                disp(['Decimal Point Out of Range/Undefined'])  %#ok<NBRAK>
                return;
            end
    
            % Get Unit Code and Convert to Actual Load
            while true
                obj.temp = str2num(char(obj.instr.Get_Unit_Code(obj.deviceHandle)));
                if ~(isempty(obj.temp))
                    break
                end
            end
            obj.unitCode = obj.temp + 1;
            obj.unitChr = obj.unitCodeData{obj.unitCode,1};
   
            %Get Normal Data
            while true
                obj.temp = str2num(char(obj.instr.Normal_Data_Request(obj.deviceHandle)));
                if ~(isempty(obj.temp))
                    break
                end
            end
            obj.normalData = obj.temp;
    
            % Calculate Output and Return the Value
    
            % Use retrieved variables to calculate measured load cell output
            obj.calculatedReading = obj.normalData - obj.offsetValue;
            obj.calculatedReading = obj.calculatedReading/(obj.fullScaleValue - obj.offsetValue);
            obj.calculatedReading = obj.calculatedReading*obj.fullScaleLoad;
            obj.calculatedReading = obj.calculatedReading/10^obj.decimalPoint;
            
            sensorOutput = obj.calculatedReading;
        end

        function resetImpl(obj) %#ok<MANU>
            % Initialize / reset discrete-state properties
        end

        function releaseImpl(obj)
            % Release resources, such as file handles
            % Close the connection to properly disconnect device and allow access by
            % other programs, otherwise you must disconnect drive from USB port.
            obj.instr.Close_Device_Connection(obj.deviceHandle);
        end
        function dataout = getOutputDataTypeImpl(~)
            dataout = 'double';
        end
        function sizeout = getOutputSizeImpl(~)
            sizeout = [1 1];
        end
        function cplxout = isOutputComplexImpl(~)
            cplxout = false;
        end
        function fixedout = isOutputFixedSizeImpl(~)
            fixedout = true;
        end
    end
end
